@@ -108,7 +108,7 @@ class WebBrowser:
 
   def launchUrl(self, url = 'https://dichvucong.baohiemxahoi.gov.vn/#/dang-ky?loaidoituong=0'):
     self.driver.get(url);
-    time.sleep(1) # Let the user actually see something!
+    time.sleep(0.5)
     self.driver.refresh()
     time.sleep(2)
 
@@ -126,7 +126,6 @@ class WebBrowser:
       self.logger.warning("Hay quay lai website")
 
   def waitElementOut(self, action_name, bytype, str_find, timeout, delay, interval):
-    self.driver.implicitly_wait(interval)
     result = None
     times  = int(timeout / interval)
     not_found_times = 0
@@ -147,7 +146,6 @@ class WebBrowser:
       except:
         result = None
         break
-    self.driver.implicitly_wait(30)
     return result
 
   def searchElement(self, element_name, bytype, str_find, timeout, delay, interval):
@@ -163,17 +161,19 @@ class WebBrowser:
           result = WebDriverWait(self.driver, interval).until(EC.visibility_of_element_located((By.CLASS_NAME,str_find)))
         elif bytype == "ID":
           result = self.driver.find_element_by_id(str_find)
-          # result = WebDriverWait(self.driver, interval).until(EC.visibility_of_element_located((By.ID,str_find)))
+          if result:
+            break
         elif bytype == "CSS":
           result = WebDriverWait(self.driver, interval).until(EC.visibility_of_element_located((By.CSS,str_find)))
-        else:
-          result = None
+
         if result and result.is_displayed() and result.is_enabled():
           break
+        else:
+          time.sleep(interval)
       except:
         result = None
         time.sleep(interval)
-    self.driver.implicitly_wait(30)
+    self.driver.implicitly_wait(20)
     return result
 
   def searchSendkeys(self, element_name, bytype, str_find, enter_keys, timeout, delay, interval):
@@ -188,11 +188,14 @@ class WebBrowser:
           break
 
   def searchClick(self,element_name, bytype, str_find, timeout, delay, interval):
-    search = self.searchElement(element_name, bytype, str_find, timeout, delay, interval)
-    if search:
-      search.click()
-      if delay > 0:
-        time.sleep(delay)
+    try:
+      search = self.searchElement(element_name, bytype, str_find, timeout, delay, interval)
+      if search:
+        search.click()
+        if delay > 0:
+          time.sleep(delay)
+    except:
+      search = None
     return search
 
   def waitElementIn(self, action_name, bytype, str_find, timeout, delay, interval):
@@ -217,32 +220,31 @@ class WebBrowser:
       except:
         result = None
         time.sleep(interval)
-    self.driver.implicitly_wait(30)
+    self.driver.implicitly_wait(20)
     return result
 
   def fillForm(self, file_cd, file_mt, file_ms, ho_ten, ma_so_str, cmnd_str, tinh_tp_str, huyen_str, xa_str, dia_chi, so_dt_str, chon_bhxh_hm=True):
     #Ho Ten
     if ho_ten:
-      self.searchSendkeys('Ho Ten', 'CLASS_NAME', 'mat-input-element', ho_ten, 15, 0, 2)
+      self.searchSendkeys('Ho Ten', 'CLASS_NAME', 'mat-input-element', ho_ten, 20, 0, 2)
 
     #click Ma so
     if ma_so_str:
-      self.searchSendkeys('Ma so', 'XPATH', '//input[@formcontrolname="MaSoBhxh"]', ma_so_str, 5, 0, 2)
+      self.searchSendkeys('Ma so', 'XPATH', '//input[@formcontrolname="MaSoBhxh"]', ma_so_str, 10, 0, 2)
 
     #fileAnhCaNhan
     if file_cd:
       if os.path.exists(file_cd):
-        search = self.searchElement("fileAnhCaNhan", 'ID', 'fileAnhCaNhan', 2, 0,2)
+        search = self.searchElement("fileAnhCaNhan", 'ID', 'fileAnhCaNhan', 10, 0,2)
         search.send_keys(file_cd)
         search = self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="body-dialog"]/mat-toolbar/section/div/button[6]', 20,0,2)
-        if search:
-          search.click()
+        search = self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="body-dialog"]/mat-toolbar/section/div/button[6]', 4,0,2)
         search = self.waitElementOut('fileAnhCaNhan','XPATH', '//*[@id="footer-dialog"]/button/span', 30,0, 2)
         if search:
-          search.click();
+          self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="footer-dialog"]/button/span', 4,0,2)
       else:
         file_cd = os.path.join(self.current_path, "fileAnhCaNhan.jpg")
-        search = self.searchElement("fileAnhCaNhan", 'ID', 'fileAnhCaNhan', 2, 0,2)
+        search = self.searchElement("fileAnhCaNhan", 'ID', 'fileAnhCaNhan', 10, 0,2)
         search.send_keys(file_cd)
         self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="body-dialog"]/mat-toolbar/section/div/button[6]', 10,0,2)
         self.searchClick('fileAnhCaNhan','XPATH', '//*[@id="footer-dialog"]/button/span', 10,0, 2)
@@ -251,15 +253,15 @@ class WebBrowser:
       #cmnd mat truoc
       if file_mt:
         if os.path.exists(file_mt):
-          search = self.searchElement("fileAnhCmndMatTruoc", 'ID', 'fileAnhCmndMatTruoc', 2, 0,2)
+          search = self.searchElement("fileAnhCmndMatTruoc", 'ID', 'fileAnhCmndMatTruoc', 10, 0,2)
           search.send_keys(file_mt)
           search = self.waitElementOut('fileAnhCmndMatTruoc', 'XPATH', '//*[@id="footer-dialog"]/button/span', 60,0, 2)
           if search:
-            search.click();
+            self.searchClick('fileAnhCmndMatTruoc', 'XPATH', '//*[@id="footer-dialog"]/button/span', 4,0,2)
         else:
           file_mt = os.path.join(self.current_path, "fileAnhCmndMatTruoc.PNG")
           if os.path.exists(file_mt):
-            search = self.searchElement("fileAnhCmndMatTruoc", 'ID', 'fileAnhCmndMatTruoc', 2, 0,2)
+            search = self.searchElement("fileAnhCmndMatTruoc", 'ID', 'fileAnhCmndMatTruoc', 10, 0,2)
             search.send_keys(file_mt)
             self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="body-dialog"]/mat-toolbar/section/div/button[6]', 10,0,2)
             self.searchClick('fileAnhCmndMatTruoc','XPATH', '//*[@id="footer-dialog"]/button/span', 10,0, 2)
@@ -267,38 +269,38 @@ class WebBrowser:
       #cmnd mat sau
       if file_ms:
         if os.path.exists(file_ms):
-          search = self.searchElement("fileAnhCmndMatSau", 'ID', 'fileAnhCmndMatSau', 2, 0,2)
+          search = self.searchElement("fileAnhCmndMatSau", 'ID', 'fileAnhCmndMatSau', 10, 0,2)
           search.send_keys(file_ms)
           search = self.waitElementOut('fileAnhCmndMatSau','XPATH', '//*[@id="footer-dialog"]/button/span', 60,0, 2)
           if search:
-            search.click();
+            self.searchClick('fileAnhCmndMatSau', 'XPATH', '//*[@id="footer-dialog"]/button/span', 4,0,2)
         else:
           file_ms = os.path.join(self.current_path, "fileAnhCmndMatSau.PNG")
           if os.path.exists(file_ms):
-            search = self.searchElement("fileAnhCmndMatSau", 'ID', 'fileAnhCmndMatSau', 2, 0,2)
+            search = self.searchElement("fileAnhCmndMatSau", 'ID', 'fileAnhCmndMatSau', 10, 0,2)
             search.send_keys(file_ms)
             self.searchClick('fileAnhCaNhan', 'XPATH', '//*[@id="body-dialog"]/mat-toolbar/section/div/button[6]', 10,0,2)
             self.searchClick('fileAnhCmndMatSau','XPATH', '//*[@id="footer-dialog"]/button/span', 10,0, 2)
 
     #cmnd_str
     if cmnd_str:
-      self.searchSendkeys('So CMND', 'XPATH', '//input[@formcontrolname="Cmnd"]', cmnd_str, 5, 0, 2)
+      self.searchSendkeys('So CMND', 'XPATH', '//input[@formcontrolname="Cmnd"]', cmnd_str, 10, 0, 2)
 
     #Tinh
     if tinh_tp_str:
-      self.searchSendkeys('Tinh', 'XPATH', '//*[@id="mat-input-8"]', tinh_tp_str, 5, 3,2)
+      self.searchSendkeys('Tinh', 'XPATH', '//*[@id="mat-input-8"]', tinh_tp_str, 10, 3,2)
     #huyen_str
     if huyen_str:
-      self.searchSendkeys('Huyen', 'XPATH', '//*[@id="mat-input-9"]', huyen_str, 5, 3,2)
+      self.searchSendkeys('Huyen', 'XPATH', '//*[@id="mat-input-9"]', huyen_str, 10, 3,2)
     #xa_str
     if xa_str:
-      self.searchSendkeys('Xa', 'XPATH', '//*[@id="mat-input-10"]', xa_str, 5, 2,2)
+      self.searchSendkeys('Xa', 'XPATH', '//*[@id="mat-input-10"]', xa_str, 10, 2,2)
     #so nha
     if dia_chi:
-      self.searchSendkeys('So Nha', 'XPATH', '//*[@id="mat-input-11"]', dia_chi, 5, 0,2)
+      self.searchSendkeys('So Nha', 'XPATH', '//*[@id="mat-input-11"]', dia_chi, 10, 0,2)
     #So di dong
     if so_dt_str:
-      self.searchSendkeys('So Di Dong', 'XPATH', '//*[@id="mat-input-3"]', so_dt_str, 5, 0,2)
+      self.searchSendkeys('So Di Dong', 'XPATH', '//*[@id="mat-input-3"]', so_dt_str, 10, 0,2)
     #Click Chon
     if chon_bhxh_hm:
       self.searchClick('Cick Chon', 'XPATH', '//*[@id="form-cnt"]/div[9]/div/div/button/span', 30,2,2)
@@ -346,7 +348,7 @@ class WebBrowser:
                 search.clear()
                 time.sleep(0.5)
                 search.send_keys(search_value)
-                time.sleep(0.5)
+                time.sleep(1)
                 break
           # Click 'Ghi Nhan'
           self.searchClick('Ghi Nhan', 'XPATH', '//*[@id="cdk-step-content-0-1"]/app-dang-ky-ca-nhan/form/div/div/div/div[3]/div/button/span', 10, 1, 2)
@@ -356,7 +358,7 @@ class WebBrowser:
 
         # Kiểm tra 'Thông Báo lỗi'
         error_msg = ""
-        search = self.searchElement('Thong Bao', 'XPATH', '//*[@id="toast-container"]/div/div[1]', 2,0, 2)
+        search = self.searchElement('Thong Bao', 'XPATH', '//*[@id="toast-container"]/div/div[1]', 2,0,2)
         if search:
           error_msg = search.text
 
@@ -442,7 +444,7 @@ class WebBrowser:
 
       if not '@' in search_value and ghi_nhan:
         # Click 'Dong'
-        search = self.searchClick('Nut Dong', 'XPATH', '//button[@aria-label="Close dialog"]', 6, 5, 2)
+        search = self.searchClick('Nut Dong', 'XPATH', '//button[@aria-label="Close dialog"]', 6, 3, 2)
         if search:
           self.logger.info("Da hoan thanh")
           status = "xong"
